using System;
using System.Drawing;
using System.Net;
using TileServer;

public abstract class OnEarth : IServer
{
    protected string ImageTypeValue;
    protected int MaximumZoomValue;
    protected string ServerNameValue;
    protected string[] URL = new string[2];

    protected OnEarth()
    {
    }

    public virtual bool DownloadTile(int X, int Y, int Zoom, string Filename)
    {
        bool flag;

		double WLON = LonFromXMerc(X, Zoom);
		double ELON = LonFromXMerc(X + 1, Zoom);
		double NLAT = LatFromYMerc(Y, Zoom);
		double SLAT = LatFromYMerc(Y+ 1, Zoom);	
				
		string _SLAT = Convert.ToString(SLAT);		
		string _WLON = Convert.ToString(WLON);
		string _NLAT = Convert.ToString(NLAT);
		string _ELON = Convert.ToString(ELON);
				
        string lBbox = _WLON + "," + _SLAT + "," + _ELON + "," + _NLAT; 
 		int index = (X + Y) % 2;
        string requestUriString = this.URL[index] + lBbox;     
        try
        {
            HttpWebRequest request = (HttpWebRequest) WebRequest.Create(requestUriString);
            request.UserAgent = "Mozilla/4.0 (compatible; MSIE 6.0;Windows NT 5.1";
            HttpWebResponse response = (HttpWebResponse) request.GetResponse();
            if (response.ContentType.IndexOf("image") > -1)
            {
                Image.FromStream(response.GetResponseStream()).Save(Filename);
                flag = true;
            }
            else
            {
                flag = false;
            }
            response.Close();
        }
        catch (Exception)
        {
            flag = false;
        }
        return flag;
    }

    protected string BaseImageType
    {
        get
        {
            return this.ImageTypeValue;
        }
        set
        {
            this.ImageTypeValue = value;
        }
    }

    protected int BaseMaximumZoom
    {
        get
        {
            return this.MaximumZoomValue;
        }
        set
        {
            this.MaximumZoomValue = value;
        }
    }

    protected string BaseServerName
    {
        get
        {
            return this.ServerNameValue;
        }
        set
        {
            this.ServerNameValue = value;
        }
    }

    protected string BaseURL0
    {
        get
        {
            return this.URL[0];
        }
        set
        {
            this.URL[0] = value;
        }
    }

    protected string BaseURL1
    {
        get
        {
            return this.URL[1];
        }
        set
        {
            this.URL[1] = value;
        }
    }

    public string ImageType
    {
        get
        {
            return this.ImageTypeValue;
        }
    }

    public int MaximumZoom
    {
        get
        {
            return this.MaximumZoomValue;
        }
    }

    public string ServerName
    {
        get
        {
            return this.ServerNameValue;
        }
    }
    
	public double LatFromYMerc(int Y, int Z)
	{
		double num = 3.14159265358979 / Math.Pow(2.0, (double) Z);
		double d = Y * num;
		d = 3.14159265358979 - d;
		d = Math.Exp(d);
		d = 2.0 * Math.Atan(d);
		return ((d - 1.570796326794895) / 0.017453292519943278);
	}

	public double LonFromXMerc(int X, int Z)
	{
		double num = 3.14159265358979 / Math.Pow(2.0, (double) Z);
		return (((X * num) - 3.14159265358979) / 0.017453292519943278);
	}    
}

